@@ -1,7 +1,7 @@
 /*
  *  Authors: Rodney Dawes <dobey@ximian.com>
  *
- *  Copyright 2003 Novell, Inc. (www.novell.com)
+ *  Copyright 2003-2004 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of version 2 of the GNU General Public License
@@ -74,87 +74,47 @@ static void wp_properties_help (GtkWindow * parent, char const * helpfile,
   }
 }
 
-static void gnome_wp_add_image (GnomeWPCapplet * capplet,
-				const gchar * filename) {
-  GnomeWPItem * item;
+static void gnome_wp_capplet_scroll_to_item (GnomeWPCapplet * capplet,
+					     GnomeWPItem * item) {
   GtkTreePath * path;
-  GdkColor color1, color2;
+
+  g_return_if_fail (capplet != NULL);
+  g_return_if_fail (item != NULL);
+
+  path = gtk_tree_row_reference_get_path (item->rowref);
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
+			    NULL, FALSE);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
+				path, NULL, TRUE, 0.5, 0.0);
+  gtk_tree_path_free (path);
+}
+
+static GnomeWPItem * gnome_wp_add_image (GnomeWPCapplet * capplet,
+					 const gchar * filename) {
+  GnomeWPItem * item;
 
   item = g_hash_table_lookup (capplet->wphash, filename);
   if (item != NULL) {
-    GtkTreePath * path;
-    
     if (item->deleted) {
       item->deleted = FALSE;
       wp_props_load_wallpaper (item->filename, item, capplet);
     }
-
-    path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-			      NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				  path, NULL, TRUE, 0.5, 0.0);
-    gtk_tree_path_free (path);
-
-    return;
-  }
-
-  item = g_new0 (GnomeWPItem, 1);
-  
-  item->filename = g_strdup (filename);
-
-  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-
-  item->shade_type = gconf_client_get_string (capplet->client,
-					      WP_SHADING_KEY, NULL);
-  item->pri_color = gconf_client_get_string (capplet->client,
-					     WP_PCOLOR_KEY, NULL);
-  item->sec_color = gconf_client_get_string (capplet->client,
-					     WP_SCOLOR_KEY, NULL);
-
-  gdk_color_parse (item->pri_color, &color1);
-  gdk_color_parse (item->sec_color, &color2);
-     
-  item->pcolor = gdk_color_copy (&color1);
-  item->scolor = gdk_color_copy (&color2);
-
-  if (!strncmp (item->fileinfo->mime_type, "image/", strlen ("image/"))) {
-    if (item->name == NULL) {
-      item->name = g_strdup (item->fileinfo->name);
-    }
-    item->options = gconf_client_get_string (capplet->client,
-					     WP_OPTIONS_KEY,
-					     NULL);
-    if (!strcmp (item->options, "none")) {
-      item->options = g_strdup ("wallpaper");
-    }
-    gnome_wp_item_update_description (item);
-     
-    g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-    wp_props_load_wallpaper (item->filename, item, capplet);
-
-    gconf_client_set_string (capplet->client, WP_FILE_KEY,
-			     item->filename, NULL);
-    gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
-			     item->options, NULL);
-
-    path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-			      NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				  path, NULL, TRUE, 0.5, 0.0);
-    gtk_tree_path_free (path);
   } else {
-    gnome_wp_item_free (item);
+    item = gnome_wp_item_new (filename, capplet->wphash, capplet->thumbs);
+    if (item != NULL) {
+      wp_props_load_wallpaper (item->filename, item, capplet);
+    }
   }
+
+  return item;
 }
 
 static void gnome_wp_add_images (GnomeWPCapplet * capplet,
-				 GSList * images,
-				 gchar ** files) {
+				 GSList * images) {
   GdkCursor * cursor;
-  gint i;
+  GnomeWPItem * item;
 
+  item = NULL;
   cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
 				       GDK_WATCH);
   gdk_window_set_cursor (capplet->window->window, cursor);
@@ -162,34 +122,26 @@ static void gnome_wp_add_images (GnomeWPCapplet * capplet,
 
   if (images != NULL) {
     for (; images != NULL; images = images->next) {
-      gnome_wp_add_image (capplet, images->data);
-    }
-  } else {
-    for (i = 0; files && files[i]; i++) {
-      gnome_wp_add_image (capplet, files[i]);
+      item = gnome_wp_add_image (capplet, images->data);
     }
   }
 
   gdk_window_set_cursor (capplet->window->window, NULL);
+
+  if (item != NULL) {
+    gnome_wp_capplet_scroll_to_item (capplet, item);
+  }
 }
 
 static void gnome_wp_file_open_dialog (GtkWidget * widget,
 				       GnomeWPCapplet * capplet) {
-#if GTK_CHECK_VERSION (2, 3, 0)
   GSList * files;
-#else
-  gchar ** files;
-#endif
 
   switch (gtk_dialog_run (GTK_DIALOG (capplet->filesel))) {
   case GTK_RESPONSE_OK:
-#if GTK_CHECK_VERSION (2, 3, 0)
     files = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (capplet->filesel));
-    gnome_wp_add_images (capplet, files, NULL);
-#else
-    files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (capplet->filesel));
-    gnome_wp_add_images (capplet, NULL, files);
-#endif
+    gnome_wp_add_images (capplet, files);
+    g_slist_free (files);
   case GTK_RESPONSE_CANCEL:
   default:
     gtk_widget_hide (capplet->filesel);
@@ -197,44 +149,6 @@ static void gnome_wp_file_open_dialog (GtkWidget * widget,
   }
 }
 
-static void bg_add_multiple_files (GnomeVFSURI * uri,
-				   GnomeWPCapplet * capplet) {
-  GnomeWPItem * item;
-  GdkColor color1, color2;
-
-  item = g_hash_table_lookup (capplet->wphash, gnome_vfs_uri_get_path (uri));
-  if (item != NULL) {
-    return;
-  }
-
-  item = g_new0 (GnomeWPItem, 1);
-
-  item->filename = gnome_vfs_unescape_string_for_display (gnome_vfs_uri_get_path (uri));
-  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-  item->name = g_strdup (item->fileinfo->name);
-  item->options = gconf_client_get_string (capplet->client,
-					   WP_OPTIONS_KEY,
-					   NULL);
-
-  item->shade_type = gconf_client_get_string (capplet->client,
-					      WP_SHADING_KEY, NULL);
-  item->pri_color = gconf_client_get_string (capplet->client,
-					     WP_PCOLOR_KEY, NULL);
-  item->sec_color = gconf_client_get_string (capplet->client,
-					     WP_SCOLOR_KEY, NULL);
-
-  gdk_color_parse (item->pri_color, &color1);
-  gdk_color_parse (item->sec_color, &color2);
-     
-  item->pcolor = gdk_color_copy (&color1);
-  item->scolor = gdk_color_copy (&color2);
-
-  gnome_wp_item_update_description (item);
-  
-  g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-  wp_props_load_wallpaper (item->filename, item, capplet);
-}
-
 static void bg_properties_dragged_image (GtkWidget * widget,
 					 GdkDragContext * context,
 					 gint x, gint y,
@@ -244,77 +158,27 @@ static void bg_properties_dragged_image (GtkWidget * widget,
 
   if (info == TARGET_URI_LIST || info == TARGET_BGIMAGE) {
     GList * uris;
+    GSList * realuris = NULL;
 
     uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
 
     if (uris != NULL && uris->data != NULL) {
-      if (g_list_length (uris) == 1) {
-	GnomeVFSURI * uri = (GnomeVFSURI *) uris->data;
-	GnomeWPItem * item;
-	GtkTreePath * path;
+      GdkCursor * cursor;
 
-	item = g_hash_table_lookup (capplet->wphash,
-				    gnome_vfs_uri_get_path (uri));
+      cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
+					   GDK_WATCH);
+      gdk_window_set_cursor (capplet->window->window, cursor);
+      gdk_cursor_unref (cursor);
 
-	if (item == NULL) {
-	  GdkColor color1, color2;
-
-	  item = g_new0 (GnomeWPItem, 1);
-
-	  item->filename = gnome_vfs_unescape_string_for_display (gnome_vfs_uri_get_path (uri));
-	  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-	  item->name = g_strdup (item->fileinfo->name);
-	  item->options = gconf_client_get_string (capplet->client,
-						   WP_OPTIONS_KEY,
-						   NULL);
-	
-	  item->shade_type = gconf_client_get_string (capplet->client,
-						      WP_SHADING_KEY, NULL);
-	  item->pri_color = gconf_client_get_string (capplet->client,
-						     WP_PCOLOR_KEY, NULL);
-	  item->sec_color = gconf_client_get_string (capplet->client,
-						     WP_SCOLOR_KEY, NULL);
-
-	  gdk_color_parse (item->pri_color, &color1);
-	  gdk_color_parse (item->sec_color, &color2);
-     
-	  item->pcolor = gdk_color_copy (&color1);
-	  item->scolor = gdk_color_copy (&color2);
-
-	  gnome_wp_item_update_description (item);
-
-	  g_hash_table_insert (capplet->wphash, g_strdup (item->filename),
-			       item);
-	  wp_props_load_wallpaper (item->filename, item, capplet);
-	} else if (item->deleted) {
-	  item->deleted = FALSE;
-	  wp_props_load_wallpaper (item->filename, item, capplet);
-	}
-	gconf_client_set_string (capplet->client, WP_FILE_KEY,
-				 item->filename, NULL);
-	gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
-				 item->options, NULL);
-
-	path = gtk_tree_row_reference_get_path (item->rowref);
-	gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				  NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				      path, NULL, TRUE, 0.5, 0.0);
-	gtk_tree_path_free (path);
-      } else if (g_list_length (uris) > 1) {
-	GdkCursor * cursor;
-
-	cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
-					     GDK_WATCH);
-	gdk_window_set_cursor (capplet->window->window, cursor);
-	gdk_cursor_unref (cursor);
-
-	g_list_foreach (uris, (GFunc) bg_add_multiple_files, capplet);
-
-	gdk_window_set_cursor (capplet->window->window, NULL);
+      for (; uris != NULL; uris = uris->next) {
+	realuris = g_slist_append (realuris,
+				   g_strdup (gnome_vfs_uri_get_path (uris->data)));
       }
+      gnome_wp_add_images (capplet, realuris);
+      gdk_window_set_cursor (capplet->window->window, NULL);
     }
     gnome_vfs_uri_list_free (uris);
+    g_slist_free (realuris);
   }
 }
 
@@ -332,6 +196,7 @@ static void wp_props_load_wallpaper (gchar * key,
   gtk_list_store_append (GTK_LIST_STORE (capplet->model), &iter);
 
   pixbuf = gnome_wp_item_get_thumbnail (item, capplet->thumbs);
+  gnome_wp_item_update_description (item);
 
   if (pixbuf != NULL) {
     gtk_list_store_set (GTK_LIST_STORE (capplet->model), &iter,
@@ -486,10 +351,12 @@ static void gnome_wp_remove_wp (gchar * key, GnomeWPItem * item,
 
   if (item->rowref != NULL && item->deleted == FALSE) {
     path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_model_get_iter (capplet->model, &iter, path);
-    gtk_tree_path_free (path);
+    if (path != NULL) {
+      gtk_tree_model_get_iter (capplet->model, &iter, path);
+      gtk_tree_path_free (path);
 
-    gtk_list_store_remove (GTK_LIST_STORE (capplet->model), &iter);
+      gtk_list_store_remove (GTK_LIST_STORE (capplet->model), &iter);
+    }
   }
 }
 
@@ -514,6 +381,7 @@ static void wallpaper_properties_clicked (GtkWidget * dialog,
     wp_properties_help (GTK_WINDOW (dialog),
 			"user-guide.xml", "goscustdesk-7");
     break;
+  case GTK_RESPONSE_DELETE_EVENT:
   case GTK_RESPONSE_CLOSE: {
     gtk_widget_destroy (dialog);
     gnome_wp_main_quit (capplet);
@@ -525,6 +393,7 @@ static void wallpaper_properties_clicked (GtkWidget * dialog,
 static void gnome_wp_scale_type_changed (GtkMenuShell * shell,
 					 GnomeWPCapplet * capplet) {
   GnomeWPItem * item = NULL;
+  GdkPixbuf * pixbuf;
   GtkTreeIter iter;
   GtkTreeModel * model;
   GtkTreeSelection * selection;
@@ -558,6 +427,11 @@ static void gnome_wp_scale_type_changed (GtkMenuShell * shell,
   default:
     break;
   }
+  pixbuf = gnome_wp_item_get_thumbnail (item, capplet->thumbs);
+  gtk_list_store_set (GTK_LIST_STORE (capplet->model), &iter,
+		      0, pixbuf,
+		      -1);
+  g_object_unref (pixbuf);
   gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
 			   item->options, NULL);
 }
@@ -706,6 +580,8 @@ static void gnome_wp_remove_wallpaper (GtkWidget * widget,
     item->deleted = TRUE;
 
     gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+
+    g_free (wpfile);
   }
   first = gtk_tree_path_new_first ();
   gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview),
@@ -717,7 +593,6 @@ static gboolean gnome_wp_load_stuffs (void * data) {
   GnomeWPCapplet * capplet = (GnomeWPCapplet *) data;
   gchar * imagepath, * style;
   GnomeWPItem * item;
-  GtkTreePath * path;
 
   style = gconf_client_get_string (capplet->client,
 				   WP_OPTIONS_KEY,
@@ -740,12 +615,7 @@ static gboolean gnome_wp_load_stuffs (void * data) {
       wp_props_load_wallpaper (item->filename, item, capplet);
     }
 
-    path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-			      NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				  path, NULL, TRUE, 0.5, 0.0);
-    gtk_tree_path_free (path);
+    gnome_wp_capplet_scroll_to_item (capplet, item);
 
     gnome_wp_option_menu_set (capplet, item->options, FALSE);
     gnome_wp_option_menu_set (capplet, item->shade_type, TRUE);
@@ -759,93 +629,16 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 				item->scolor->green,
 				item->scolor->blue, 65535);
   } else if (strcmp (style, "none") != 0) {
-    GdkColor color1, color2;
-
-    item = g_new0 (GnomeWPItem, 1);
-
-    item->filename = g_strdup (imagepath);
-
-    item->shade_type = gconf_client_get_string (capplet->client,
-						WP_SHADING_KEY, NULL);
-    item->pri_color = gconf_client_get_string (capplet->client,
-					       WP_PCOLOR_KEY, NULL);
-    item->sec_color = gconf_client_get_string (capplet->client,
-					       WP_SCOLOR_KEY, NULL);
-
-    gdk_color_parse (item->pri_color, &color1);
-    gdk_color_parse (item->sec_color, &color2);
-
-    item->pcolor = gdk_color_copy (&color1);
-    item->scolor = gdk_color_copy (&color2);
-
-    if (g_file_test (item->filename, G_FILE_TEST_EXISTS)) {
-      item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-      item->name = g_strdup (item->fileinfo->name);
-      item->options = gconf_client_get_string (capplet->client,
-					       WP_OPTIONS_KEY,
-					       NULL);
-      item->deleted = FALSE;
-
-      gnome_wp_item_update_description (item);
-
-      g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-      wp_props_load_wallpaper (item->filename, item, capplet);
-
-      path = gtk_tree_row_reference_get_path (item->rowref);
-      gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				    path, NULL, TRUE, 0.5, 0.0);
-      gtk_tree_path_free (path);
-
-      gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (capplet->pc_picker),
-				  item->pcolor->red,
-				  item->pcolor->green,
-				  item->pcolor->blue, 65535);
-      gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (capplet->sc_picker),
-				  item->scolor->red,
-				  item->scolor->green,
-				  item->scolor->blue, 65535);
-
-      gnome_wp_option_menu_set (capplet, item->options, FALSE);
-      gnome_wp_option_menu_set (capplet, item->shade_type, TRUE);
-    } else {
-      gnome_wp_item_free (item);
-    }
+    item = gnome_wp_add_image (capplet, imagepath);
+    gnome_wp_capplet_scroll_to_item (capplet, item);
   }
 
   item = g_hash_table_lookup (capplet->wphash, "(none)");
   if (item == NULL) {
-    GdkColor color1, color2;
-
-    item = g_new0 (GnomeWPItem, 1);
-
-    item->deleted = FALSE;
-    item->filename = g_strdup ("(none)");
-
-    item->shade_type = gconf_client_get_string (capplet->client,
-						WP_SHADING_KEY, NULL);
-    item->pri_color = gconf_client_get_string (capplet->client,
-					       WP_PCOLOR_KEY, NULL);
-    item->sec_color = gconf_client_get_string (capplet->client,
-					       WP_SCOLOR_KEY, NULL);
-
-    gdk_color_parse (item->pri_color, &color1);
-    gdk_color_parse (item->sec_color, &color2);
-
-    item->pcolor = gdk_color_copy (&color1);
-    item->scolor = gdk_color_copy (&color2);
-
-    item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-    item->name = g_strdup (item->fileinfo->name);
-    item->options = gconf_client_get_string (capplet->client,
-					     WP_OPTIONS_KEY,
-					     NULL);
-
-    gnome_wp_item_update_description (item);
-
-    g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-    wp_props_load_wallpaper (item->filename, item, capplet);
+    item = gnome_wp_item_new ("(none)", capplet->wphash, capplet->thumbs);
+    if (item != NULL) {
+      wp_props_load_wallpaper (item->filename, item, capplet);
+    }
   } else {
     if (item->deleted == TRUE) {
       item->deleted = FALSE;
@@ -853,17 +646,17 @@ static gboolean gnome_wp_load_stuffs (void * data) {
     }
 
     if (!strcmp (style, "none")) {
-      path = gtk_tree_row_reference_get_path (item->rowref);
-      gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				    path, NULL, TRUE, 0.5, 0.0);
-      gtk_tree_path_free (path);
+      gnome_wp_capplet_scroll_to_item (capplet, item);
     }
   }
   g_free (imagepath);
   g_free (style);
 
+  if (capplet->uri_list) {
+    gnome_wp_add_images (capplet, capplet->uri_list);
+    g_slist_free (capplet->uri_list);
+  }
+
   return FALSE;
 }
 
@@ -872,17 +665,25 @@ static gint gnome_wp_list_sort (GtkTreeModel * model,
 				GnomeWPCapplet * capplet) {
   gchar * foo, * bar;
   gchar * desca, * descb;
+  gint retval;
 
   gtk_tree_model_get (model, a, 1, &desca, 2, &foo, -1);
   gtk_tree_model_get (model, b, 1, &descb, 2, &bar, -1);
 
   if (!strcmp (foo, "(none)")) {
-    return -1;
+    retval =  -1;
   } else if (!strcmp (bar, "(none)")) {
-    return 1;
+    retval =  1;
   } else {
-    return strcmp (desca, descb);
+    retval = g_utf8_collate (desca, descb);
   }
+
+  g_free (desca);
+  g_free (descb);
+  g_free (foo);
+  g_free (bar);
+
+  return retval;
 }
 
 static void gnome_wp_file_changed (GConfClient * client, guint id,
@@ -891,7 +692,6 @@ static void gnome_wp_file_changed (GConfClient * client, guint id,
   GtkTreeSelection * selection;
   GtkTreeModel * model;
   GtkTreeIter iter;
-  GtkTreePath * path;
   GnomeWPItem * item;
   gchar * wpfile, * selected;
 
@@ -903,53 +703,10 @@ static void gnome_wp_file_changed (GConfClient * client, guint id,
     gtk_tree_model_get (model, &iter, 2, &selected, -1);
     if (strcmp (selected, wpfile) != 0) {
       if (item != NULL) {
-	path = gtk_tree_row_reference_get_path (item->rowref);
-	gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				  NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				      path, NULL, TRUE, 0.5, 0.0);
-	gtk_tree_path_free (path);
+	gnome_wp_capplet_scroll_to_item (capplet, item);
       } else {
-	GdkColor color1, color2;
-
-	item = g_new0 (GnomeWPItem, 1);
-
-	item->filename = g_strdup (wpfile);
-
-	item->shade_type = gconf_client_get_string (capplet->client,
-						    WP_SHADING_KEY, NULL);
-	item->pri_color = gconf_client_get_string (capplet->client,
-						   WP_PCOLOR_KEY, NULL);
-	item->sec_color = gconf_client_get_string (capplet->client,
-						   WP_SCOLOR_KEY, NULL);
-    
-	gdk_color_parse (item->pri_color, &color1);
-	gdk_color_parse (item->sec_color, &color2);
-    
-	item->pcolor = gdk_color_copy (&color1);
-	item->scolor = gdk_color_copy (&color2);
-    
-	if (g_file_test (item->filename, G_FILE_TEST_EXISTS)) {
-	  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-	  item->name = g_strdup (item->fileinfo->name);
-	  item->options = gconf_client_get_string (capplet->client,
-						   WP_OPTIONS_KEY,
-						   NULL);
-      
-	  gnome_wp_item_update_description (item);
-	  g_hash_table_insert (capplet->wphash,
-			       g_strdup (item->filename), item);
-	  wp_props_load_wallpaper (item->filename, item, capplet);
-      
-	  path = gtk_tree_row_reference_get_path (item->rowref);
-	  gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				    NULL, FALSE);
-	  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-					path, NULL, TRUE, 0.5, 0.0);
-	  gtk_tree_path_free (path);
-	} else {
-	  gnome_wp_item_free (item);
-	}
+	item = gnome_wp_add_image (capplet, wpfile);
+	gnome_wp_capplet_scroll_to_item (capplet, item);
       }
     }
     g_free (wpfile);
@@ -1051,7 +808,7 @@ static void gnome_wp_icon_theme_changed (GnomeIconTheme * theme,
   gchar * icofile;
 
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
-					  "background-capplet",
+					  "gnome-settings-background",
 					  48, NULL, NULL);
   if (icofile != NULL) {
     pixbuf = gdk_pixbuf_new_from_file (icofile, NULL);
@@ -1130,7 +887,45 @@ static GladeXML * gnome_wp_create_dialog (void) {
   return new;
 }
 
-static void wallpaper_properties_init (void) {
+static void set_accessible_name (GtkWidget *widget, const gchar *name) {
+  AtkObject *obj;
+ 
+  obj = gtk_widget_get_accessible (widget);
+  if (!GTK_IS_ACCESSIBLE (obj))
+    return;
+  if (name)
+    atk_object_set_name (obj, name);
+} 
+
+static void gnome_wp_update_preview (GtkFileChooser *chooser,
+				     GnomeWPCapplet *capplet) {
+  gchar *uri;
+
+  uri = gtk_file_chooser_get_preview_uri (chooser);
+
+  if (uri) {
+    GdkPixbuf *pixbuf;
+    gchar *mime_type;
+
+    mime_type = gnome_vfs_get_mime_type (uri);
+    pixbuf = gnome_thumbnail_factory_generate_thumbnail (capplet->thumbs,
+							 uri,
+							 mime_type);
+
+    if(pixbuf != NULL) {
+      gtk_image_set_from_pixbuf (GTK_IMAGE (capplet->image), pixbuf);
+      g_object_unref (pixbuf);
+    } else {
+      gtk_image_set_from_stock (GTK_IMAGE (capplet->image),
+				"gtk-dialog-question",
+				GTK_ICON_SIZE_DIALOG);
+    }
+    g_free (mime_type);
+  }
+  gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
+}
+  
+static void wallpaper_properties_init (poptContext ctx) {
   GnomeWPCapplet * capplet;
   GladeXML * dialog;
   GtkWidget * menu, * label;
@@ -1142,10 +937,10 @@ static void wallpaper_properties_init (void) {
   GdkPixbuf * pixbuf;
   GdkCursor * cursor;
   gchar * icofile;
+  const gchar ** args;
 
   gtk_rc_parse_string ("style \"wp-tree-defaults\" {\n"
 		       "  GtkTreeView::horizontal-separator = 6\n"
-		       "  GtkTreeView::vertical-separator = 6\n"
 		       "} widget_class \"*TreeView*\""
 		       " style \"wp-tree-defaults\"\n\n"
 		       "style \"wp-dialog-defaults\" {\n"
@@ -1164,9 +959,9 @@ static void wallpaper_properties_init (void) {
 					 WP_DELAY_KEY,
 					 NULL);
   gconf_client_add_dir (capplet->client, WP_KEYBOARD_PATH,
-			GCONF_CLIENT_PRELOAD_NONE, NULL);
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (capplet->client, WP_PATH_KEY,
-			GCONF_CLIENT_PRELOAD_NONE, NULL);
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
   gconf_client_notify_add (capplet->client,
 			   WP_DELAY_KEY,
@@ -1209,7 +1004,7 @@ static void wallpaper_properties_init (void) {
   capplet->window = glade_xml_get_widget (dialog,"gnome_wp_properties");
 
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
-					  "background-capplet",
+					  "gnome-settings-background",
 					  48, NULL, NULL);
   if (icofile != NULL) {
     pixbuf = gdk_pixbuf_new_from_file (icofile, NULL);
@@ -1266,6 +1061,7 @@ static void wallpaper_properties_init (void) {
 
   menu = gtk_menu_new ();
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Centered")); 
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-center",
 					  16, NULL, NULL);
@@ -1289,6 +1085,7 @@ static void wallpaper_properties_init (void) {
   gtk_widget_show (mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Fill Screen"));
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-fill",
 					  16, NULL, NULL);
@@ -1311,6 +1108,7 @@ static void wallpaper_properties_init (void) {
   gtk_widget_show (mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Scaled"));
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-scale",
 					  16, NULL, NULL);
@@ -1323,6 +1121,8 @@ static void wallpaper_properties_init (void) {
     gtk_box_pack_start (GTK_BOX (mbox), capplet->sitem, FALSE, FALSE, 0);
     gtk_widget_show (capplet->sitem);
   }
+  g_free (icofile);
+
   label = gtk_label_new (_("Scaled"));
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_box_pack_start (GTK_BOX (mbox), label, TRUE, TRUE, 0);
@@ -1331,6 +1131,7 @@ static void wallpaper_properties_init (void) {
   gtk_widget_show (mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Tiled"));
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-tile",
 					  16, NULL, NULL);
@@ -1343,6 +1144,8 @@ static void wallpaper_properties_init (void) {
     gtk_box_pack_start (GTK_BOX (mbox), capplet->witem, FALSE, FALSE, 0);
     gtk_widget_show (capplet->witem);
   }
+  g_free (icofile);
+
   label = gtk_label_new (_("Tiled"));
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_box_pack_start (GTK_BOX (mbox), label, TRUE, TRUE, 0);
@@ -1367,6 +1170,7 @@ static void wallpaper_properties_init (void) {
 
   menu = gtk_menu_new ();
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Solid Color"));
   mbox = gtk_hbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (mitem), mbox);
   gtk_widget_show (mbox);
@@ -1383,6 +1187,7 @@ static void wallpaper_properties_init (void) {
   gtk_menu_append (GTK_MENU (menu), mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Horizontal Gradient"));
   mbox = gtk_hbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (mitem), mbox);
   gtk_widget_show (mbox);
@@ -1399,6 +1204,7 @@ static void wallpaper_properties_init (void) {
   gtk_menu_append (GTK_MENU (menu), mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Vertical Gradient"));
   mbox = gtk_hbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (mitem), mbox);
   gtk_widget_show (mbox);
@@ -1436,6 +1242,17 @@ static void wallpaper_properties_init (void) {
   gdk_window_set_cursor (capplet->window->window, cursor);
   gdk_cursor_unref (cursor);
 
+  args = poptGetArgs (ctx);
+  if (args != NULL) {
+    const gchar ** p;
+
+    for (p = args; *p != NULL; p++) {
+      capplet->uri_list = g_slist_append (capplet->uri_list, (gchar *) *p);
+    }
+  }
+
+  poptFreeContext (ctx);
+
   g_idle_add (gnome_wp_load_stuffs, capplet);
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (capplet->treeview));
@@ -1443,8 +1260,7 @@ static void wallpaper_properties_init (void) {
 		    G_CALLBACK (gnome_wp_props_wp_selected), capplet);
 
   /* Create the file chooser dialog stuff here */
-#if GTK_CHECK_VERSION (2, 3, 0)
-  capplet->filesel = gtk_file_chooser_dialog_new (_("Add Wallpapers"),
+  capplet->filesel = gtk_file_chooser_dialog_new (_("Add Wallpaper"),
 						  GTK_WINDOW (capplet->window),
 						  GTK_FILE_CHOOSER_ACTION_OPEN,
 						  GTK_STOCK_CANCEL,
@@ -1454,15 +1270,25 @@ static void wallpaper_properties_init (void) {
 						  NULL);
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (capplet->filesel),
 					TRUE);
-#else
-  capplet->filesel = gtk_file_selection_new (_("Add Wallpapers"));
-  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (capplet->filesel),
-					  TRUE);
-#endif
+
+  gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (capplet->filesel),
+					  FALSE);
+
+  capplet->image = gtk_image_new ();
+  gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (capplet->filesel),
+				       capplet->image);
+  gtk_widget_set_size_request (capplet->image, 128, -1);
+  
+  gtk_widget_show (capplet->image);
+
+  g_signal_connect (capplet->filesel, "update-preview",
+		    G_CALLBACK (gnome_wp_update_preview), capplet);
 }
 
 gint main (gint argc, gchar *argv[]) {
-  GnomeProgram * proggie;
+  GnomeProgram * program;
+  GValue context = { 0 };
+  poptContext ctx;
 
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -1470,11 +1296,16 @@ gint main (gint argc, gchar *argv[]) {
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  proggie = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
+  program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
 				argc, argv, GNOME_PARAM_POPT_TABLE,
 				NULL, NULL);
 
-  wallpaper_properties_init ();
+  g_object_get_property (G_OBJECT (program), GNOME_PARAM_POPT_CONTEXT,
+			 g_value_init (&context, G_TYPE_POINTER));
+  ctx = g_value_get_pointer (&context);
+
+  wallpaper_properties_init (ctx);
+
   gtk_main ();
   
   return 0;
