@@ -328,6 +328,8 @@ gconf_property_editor_finalize (GObject *object)
 	g_return_if_fail (IS_GCONF_PROPERTY_EDITOR (object));
 
 	gconf_property_editor = GCONF_PROPERTY_EDITOR (object);
+
+	g_free (gconf_property_editor->p->key);
 	
 	if (gconf_property_editor->p->data_free_cb)
 		gconf_property_editor->p->data_free_cb (gconf_property_editor->p->data);
@@ -1116,8 +1118,13 @@ peditor_numeric_range_widget_changed (GConfPropertyEditor *peditor,
 							      NULL);
 	if (default_value)
 		value_wid = gconf_value_new (default_value->type);
-	else
-		value_wid = gconf_value_new (GCONF_VALUE_FLOAT);
+	else {
+		g_warning ("Unable to find a default value for key for %s.\n"
+			   "I'll assume it is an integer, but that may break things.\n"
+			   "Please be sure that the associated schema is installed",
+			   peditor->p->key);
+		value_wid = gconf_value_new (GCONF_VALUE_INT);
+	}
 
 	gconf_value_free (default_value);
 
