@@ -623,7 +623,8 @@ run_revert_dialog (struct DisplayInfo *info,
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 12);
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  gtk_window_set_title (GTK_WINDOW (dialog), "");
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Keep Resolution"));
+  gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_CENTER_ALWAYS);
   
   label = gtk_label_new (NULL);
   str = g_strdup_printf ("<b>%s</b>", _("Do you want to keep this resolution?"));
@@ -757,8 +758,10 @@ cb_dialog_response (GtkDialog *dialog, gint response_id, struct DisplayInfo *inf
 	  
       if (apply_config (info))
 	{
+	  gtk_widget_hide(GTK_WIDGET(dialog));
 	  if (!run_revert_dialog (info, GTK_WIDGET (dialog)))
 	    {
+	      gtk_widget_show(GTK_WIDGET(dialog));
 	      revert_config (info);
 	      return;
 	    }
@@ -800,7 +803,7 @@ main (int argc, char *argv[])
       XRRQueryVersion (xdisplay, &major, &minor) == 0)
     {
       GtkWidget *msg_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-						   _("The XServer does not support the XRandR extension.  Runtime resolution changes to the display size are not available."));
+						   _("The X Server does not support the XRandR extension.  Runtime resolution changes to the display size are not available."));
       gtk_dialog_run (GTK_DIALOG (msg_dialog));
       gtk_widget_destroy (msg_dialog);
       exit (0);
@@ -817,7 +820,7 @@ main (int argc, char *argv[])
   info = read_display_info (display);
   dialog = create_dialog (info);
 
-  gtk_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (cb_dialog_response), info);
+  g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (cb_dialog_response), info);
   gtk_widget_show (dialog);
 
   gtk_main ();
