@@ -30,7 +30,7 @@
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 
-#include "libgswitchit/gswitchit_xkb_config.h"
+#include "libgswitchit/gswitchit_config.h"
 
 #include "capplet-util.h"
 #include "gconf-property-editor.h"
@@ -40,9 +40,9 @@
 
 #include "gnome-keyboard-properties-xkb.h"
 
-#define CWID(s) glade_xml_get_widget (chooserDialog, s)
+static GSwitchItKbdConfig initialConfig;
 
-static GSwitchItXkbConfig initialConfig;
+GConfClient *xkbGConfClient;
 
 char *
 xci_desc_to_utf8 (XklConfigItem * ci)
@@ -101,17 +101,19 @@ model_to_widget (GConfPropertyEditor * peditor, GConfValue * value)
 static void
 cleanup_xkb_tabs (GladeXML * dialog)
 {
-  GSwitchItXkbConfigTerm (&initialConfig);
+  GSwitchItKbdConfigTerm (&initialConfig);
   XklConfigFreeRegistry ();
   XklConfigTerm ();
   XklTerm ();
+  g_object_unref (xkbGConfClient);
+  xkbGConfClient = NULL;
 }
 
 static void
 reset_to_defaults (GtkWidget * button, GladeXML * dialog)
 {
-  gconf_client_set_bool (gconf_client_get_default (),
-			 GSWITCHIT_CONFIG_XKB_KEY_OVERRIDE_SETTINGS,
+  gconf_client_set_bool (xkbGConfClient,
+			 GSWITCHIT_KBD_CONFIG_KEY_OVERRIDE_SETTINGS,
 			 TRUE, NULL);
   /* all the rest is g-s-d's business */
 }
@@ -124,90 +126,86 @@ update_model (GConfClient * client,
 }
 
 static void
-notebook_page_changed (GtkNotebook * notebook,
-                       GtkNotebookPage * page,
-                       guint pageNum,
-                       GladeXML * dialog)
+chk_separate_group_per_window_toggled (GConfPropertyEditor *peditor, 
+                                       const gchar *key, 
+                                       const GConfValue *value, 
+                                       GladeXML* dialog)
 {
-  switch (pageNum)
-  {
-    case 1:
-    case 2:
-      break;
-    default:
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("enable_preview")), FALSE);
-  }
+  xkb_layouts_enable_disable_default (dialog, value && gconf_value_get_bool (value));
 }
 
 void
 setup_xkb_tabs (GladeXML * dialog, GConfChangeSet * changeset)
 {
-  GConfClient *confClient = gconf_client_get_default ();
+  xkbGConfClient = gconf_client_get_default ();
+  GObject * peditor;
 
   XklInit (GDK_DISPLAY ());
   XklConfigInit ();
   XklConfigLoadRegistry ();
 
-//  fill_models_option_menu (dialog);
-
   gconf_peditor_new_string
-    (changeset, (gchar *) GSWITCHIT_CONFIG_XKB_KEY_MODEL,
+    (changeset, (gchar *) GSWITCHIT_KBD_CONFIG_KEY_MODEL,
      WID ("xkb_model"),
      "conv-to-widget-cb", model_to_widget,
      "conv-from-widget-cb", model_from_widget, NULL);
 
-  fill_available_layouts_tree (dialog);
-  fill_available_options_tree (dialog);
-  prepare_selected_layouts_tree (dialog);
-  prepare_selected_options_tree (dialog);
-  fill_selected_layouts_tree (dialog);
-  fill_selected_options_tree (dialog);
+  peditor = gconf_peditor_new_boolean
+    (changeset, (gchar *) GSWITCHIT_CONFIG_KEY_GROUP_PER_WINDOW, 
+	WID ("chk_separate_group_per_window"), NULL);
 
-  register_layouts_buttons_handlers (dialog);
-  register_options_buttons_handlers (dialog);
+  g_signal_connect (peditor, "value-changed", (GCallback)chk_separate_group_per_window_toggled , dialog);
+
+/* tab 2 */
+  /*fill_available_layouts_tree (dialog);*/
+  xkb_layouts_prepare_selected_tree (dialog, changeset);
+  xkb_layouts_fill_selected_tree (dialog);
+
+/* tab 3 */
+  xkb_options_fill_available_tree (dialog);
+  xkb_options_prepare_selected_tree (dialog);
+  xkb_options_fill_selected_tree (dialog);
+
+  xkb_layouts_register_buttons_handlers (dialog);
+  xkb_options_register_buttons_handlers (dialog);
   g_signal_connect (G_OBJECT (WID ("xkb_reset_to_defaults")), "clicked",
 		    G_CALLBACK (reset_to_defaults), dialog);
 
   g_signal_connect_swapped (G_OBJECT (WID ("xkb_model_pick")), "clicked",
 		            G_CALLBACK (choose_model), dialog);
 
-  register_layouts_gconf_listener (dialog);
-  register_options_gconf_listener (dialog);
+  xkb_layouts_register_gconf_listener (dialog);
+  xkb_options_register_gconf_listener (dialog);
 
   g_signal_connect (G_OBJECT (WID ("keyboard_dialog")),
 		    "destroy", G_CALLBACK (cleanup_xkb_tabs), dialog);
 
-  gconf_client_notify_add (gconf_client_get_default (),
-			   GSWITCHIT_CONFIG_XKB_KEY_MODEL,
+  gconf_client_notify_add (xkbGConfClient,
+			   GSWITCHIT_KBD_CONFIG_KEY_MODEL,
 			   (GConfClientNotifyFunc)
 			   update_model, dialog, NULL, NULL);
 
-  GSwitchItXkbConfigInit (&initialConfig, confClient);
-  g_object_unref (confClient);
-  GSwitchItXkbConfigLoadInitial (&initialConfig);
+  GSwitchItKbdConfigInit (&initialConfig, xkbGConfClient);
+  GSwitchItKbdConfigLoadFromXInitial (&initialConfig);
 
   enable_disable_restoring (dialog);
-
-  g_signal_connect_swapped (G_OBJECT (WID ("enable_preview")), "toggled",
-		            G_CALLBACK (preview_toggled), dialog);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("enable_preview")), FALSE);
-  g_signal_connect (G_OBJECT (WID ("keyboard_notebook")), "switch-page",
-		    G_CALLBACK (notebook_page_changed), dialog);
+  xkb_layouts_enable_disable_default (dialog, 
+                                      gconf_client_get_bool (xkbGConfClient, 
+                                                             GSWITCHIT_CONFIG_KEY_GROUP_PER_WINDOW, 
+                                                             NULL));
 }
 
 void
 enable_disable_restoring (GladeXML * dialog)
 {
-  GSwitchItXkbConfig gswic;
-  GConfClient *confClient = gconf_client_get_default ();
+  GSwitchItKbdConfig gswic;
   gboolean enable;
 
-  GSwitchItXkbConfigInit (&gswic, confClient);
-  g_object_unref (confClient);
-  GSwitchItXkbConfigLoad (&gswic);
+  GSwitchItKbdConfigInit (&gswic, xkbGConfClient);
+  GSwitchItKbdConfigLoadFromGConf (&gswic);
 
-  enable = !GSwitchItXkbConfigEquals (&gswic, &initialConfig);
+  enable = !GSwitchItKbdConfigEquals (&gswic, &initialConfig);
 
-  GSwitchItXkbConfigTerm (&gswic);
+  GSwitchItKbdConfigTerm (&gswic);
   gtk_widget_set_sensitive (WID ("xkb_reset_to_defaults"), enable);
 }
