@@ -26,23 +26,29 @@
 # include "config.h"
 #endif
 
-#include <glib/gstrfuncs.h>
+//#include <glib/gstrfuncs.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gconf/gconf-client.h>
 
 #include <string.h>
+#include <time.h>
 
 #include <libgnome/gnome-i18n.h>
 
 #include <libxklavier/xklavier.h>
 #include <libxklavier/xklavier_config.h>
-#include <libgswitchit/gswitchit_xkb_config.h>
+#include <libgswitchit/gswitchit_config.h>
 
 #include "gnome-settings-keyboard-xkb.h"
 #include "gnome-settings-daemon.h"
 
-static GSwitchItXkbConfig gswic;
+static GSwitchItConfig currentConfig;
+static GSwitchItKbdConfig currentKbdConfig;
+
+/* never terminated */
+static GSwitchItKbdConfig initialSysKbdConfig;
+
 static gboolean initedOk;
 
 static PostActivationCallback paCallback = NULL;
@@ -56,6 +62,23 @@ typedef enum {
 	RESPONSE_USE_GNOME
 } SysConfigChangedMsgResponse;
 
+#define noGSDKX
+
+#ifdef GSDKX
+static FILE *logfile;
+
+static void
+gnome_settings_keyboard_log_appender (const char file[], const char function[],
+                      int level, const char format[], va_list args)
+{
+        time_t now = time (NULL);
+        fprintf (logfile, "[%08ld,%03d,%s:%s/] \t", now,
+               level, file, function);
+        vfprintf (logfile, format, args);
+	fflush(logfile);
+}
+#endif
+
 static void
 activation_error (void)
 {
@@ -71,10 +94,14 @@ activation_error (void)
 							     GTK_BUTTONS_CLOSE,
 							     _
 							     ("Error activating XKB configuration.\n"
-							      "Probably internal X server problem.\n\nX server version data:\n%s\n%d\n%s\n"
+							      "It can have under various circumstances:\n"
+							      "- a bug in libxklavier library\n"
+							      "- a bug in X server (xkbcomp, xmodmap utilities)\n"
+							      "- X server with incompatible libxkbfile implemenation\n\n"
+							      "X server version data:\n%s\n%d\n%s\n"
 							      "If you report this situation as a bug, please include:\n"
 							      "- The result of <b>xprop -root | grep XKB</b>\n"
-							      "- The result of <b>gconftool-2 -R /desktop/gnome/peripherals/keyboard/xkb</b>"),
+							      "- The result of <b>gconftool-2 -R /desktop/gnome/peripherals/keyboard/kbd</b>"),
 							     vendor,
 							     release,
 							     badXFree430Release
@@ -91,76 +118,92 @@ activation_error (void)
 
 static void
 apply_settings (void)
+{
+	if (!initedOk)
+		return;
+
+	GSwitchItConfigLoadFromGConf (&currentConfig);
+	/* again, probably it would be nice to compare things 
+	   before activating them */
+	GSwitchItConfigActivate (&currentConfig);
+}
+
+static void
+apply_xkb_settings (void)
 {
 	GConfClient *confClient;
+	GSwitchItKbdConfig currentSysKbdConfig;
 
 	if (!initedOk)
 		return;
 
 	confClient = gconf_client_get_default ();
-	GSwitchItXkbConfigInit (&gswic, confClient);
+	GSwitchItKbdConfigInit (&currentSysKbdConfig, confClient);
 	g_object_unref (confClient);
-	GSwitchItXkbConfigLoad (&gswic);
 
-	if (gswic.overrideSettings) {
+	GSwitchItKbdConfigLoadFromGConf (&currentKbdConfig);
+
+	if (currentKbdConfig.overrideSettings) {
 		/* initialization - from the system settings */
-		GSwitchItXkbConfigLoadInitial (&gswic);
-		gswic.overrideSettings = FALSE;
-		GSwitchItXkbConfigSave (&gswic);
+		GSwitchItKbdConfigSaveToGConf (&initialSysKbdConfig);
 	} else {
-		if (GSwitchItXkbConfigActivate (&gswic)) {
-			if (paCallback != NULL) {
-				(*paCallback) (paCallbackUserData);
+		GSwitchItKbdConfigLoadFromXCurrent (&currentSysKbdConfig);
+		/* Activate - only if different! */
+		if (!GSwitchItKbdConfigEquals (&currentKbdConfig, &currentSysKbdConfig))
+		{
+			if (GSwitchItKbdConfigActivate (&currentKbdConfig)) {
+				if (paCallback != NULL) {
+					(*paCallback) (paCallbackUserData);
+				}
+			} else {
+				g_warning
+				    ("Could not activate the XKB configuration");
+				activation_error ();
 			}
-		} else {
-			g_warning
-			    ("Could not activate the XKB configuration");
-			activation_error ();
-		}
+		} else
+			XklDebug (100, "Actual KBD configuration was not changed: redundant notification\n");
 	}
 
-	GSwitchItXkbConfigTerm (&gswic);
+	GSwitchItKbdConfigTerm (&currentSysKbdConfig);
 }
 
 static void
 gnome_settings_keyboard_xkb_sysconfig_changed_response (GtkDialog * dialog,
 							SysConfigChangedMsgResponse
-							what2do,
-							GSwitchItXkbConfig
-							* pgswicNow)
+							what2do)
 {
 	switch (what2do) {
 	case RESPONSE_USE_X:
-		GSwitchItXkbConfigSave (pgswicNow);
+		GSwitchItKbdConfigSaveToGConf (&initialSysKbdConfig);
 		break;
 	case RESPONSE_USE_GNOME:
 		/* Do absolutely nothing - just keep things the way they are */
 		break;
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-	GSwitchItXkbConfigTerm (pgswicNow);
-	g_free (pgswicNow);
 }
 
 static void
 gnome_settings_keyboard_xkb_analyze_sysconfig (void)
 {
 	GConfClient *confClient;
-	GSwitchItXkbConfig gswicWas, *pgswicNow;
+	GSwitchItKbdConfig backupGConfKbdConfig;
+	gboolean isConfigChanged;
 
 	if (!initedOk)
 		return;
-	pgswicNow = g_new (GSwitchItXkbConfig, 1);
 	confClient = gconf_client_get_default ();
-	GSwitchItXkbConfigInit (&gswicWas, confClient);
-	GSwitchItXkbConfigInit (pgswicNow, confClient);
+	GSwitchItKbdConfigInit (&backupGConfKbdConfig, confClient);
+	GSwitchItKbdConfigInit (&initialSysKbdConfig, confClient);
 	g_object_unref (confClient);
-	GSwitchItXkbConfigLoadSysBackup (&gswicWas);
-	GSwitchItXkbConfigLoadInitial (pgswicNow);
+	GSwitchItKbdConfigLoadFromGConfBackup (&backupGConfKbdConfig);
+	GSwitchItKbdConfigLoadFromXInitial (&initialSysKbdConfig);
+	initialSysKbdConfig.overrideSettings = FALSE;
 
+	isConfigChanged = g_slist_length (backupGConfKbdConfig.layouts) &&
+	    !GSwitchItKbdConfigEquals (&initialSysKbdConfig, &backupGConfKbdConfig);
 	/* config was changed!!! */
-	if (g_slist_length (gswicWas.layouts) &&
-	    !GSwitchItXkbConfigEquals (pgswicNow, &gswicWas)) {
+	if (isConfigChanged) {
 		GtkWidget *msg = gtk_message_dialog_new_with_markup (NULL,
 								     0,
 								     GTK_MESSAGE_INFO,
@@ -179,37 +222,37 @@ gnome_settings_keyboard_xkb_analyze_sysconfig (void)
 		g_signal_connect (msg, "response",
 				  G_CALLBACK
 				  (gnome_settings_keyboard_xkb_sysconfig_changed_response),
-				  pgswicNow);
+				  NULL);
 		gtk_widget_show (msg);
 	}
-	GSwitchItXkbConfigSaveSysBackup (pgswicNow);
-	GSwitchItXkbConfigTerm (&gswicWas);
+	GSwitchItKbdConfigSaveToGConfBackup (&initialSysKbdConfig);
+	GSwitchItKbdConfigTerm (&backupGConfKbdConfig);
 }
 
 static void
 gnome_settings_keyboard_xkb_chk_lcl_xmm_response (GtkDialog * dlg,
 						  gint response)
 {
-	GConfClient *gcc = gconf_client_get_default ();
+	GConfClient *confClient = gconf_client_get_default ();
 	switch (response) {
 	case GTK_RESPONSE_OK:
-		gconf_client_set_bool (gcc, DISABLE_XMM_WARNING_KEY, TRUE,
+		gconf_client_set_bool (confClient, DISABLE_XMM_WARNING_KEY, TRUE,
 				       NULL);
 		break;
 	}
-	g_object_unref (G_OBJECT (gcc));
+	g_object_unref (G_OBJECT (confClient));
 	gtk_widget_destroy (GTK_WIDGET (dlg));
 }
 
 static void
 gnome_settings_keyboard_xkb_chk_lcl_xmm (void)
 {
-	GConfClient *gcc = gconf_client_get_default ();
+	GConfClient *confClient = gconf_client_get_default ();
 	gboolean disableWarning =
-	    gconf_client_get_bool (gcc, DISABLE_XMM_WARNING_KEY, NULL);
+	    gconf_client_get_bool (confClient, DISABLE_XMM_WARNING_KEY, NULL);
 	GDir *homeDir;
 	G_CONST_RETURN gchar *fname;
-	g_object_unref (G_OBJECT (gcc));
+	g_object_unref (G_OBJECT (confClient));
 	if (disableWarning)
 		return;
 
@@ -245,22 +288,59 @@ gnome_settings_keyboard_xkb_set_post_activation_callback
 	paCallbackUserData = userData;
 }
 
+static GdkFilterReturn
+gnome_settings_keyboard_xkb_evt_filter (GdkXEvent * xev,
+                           GdkEvent * event)
+{
+        XEvent *xevent = (XEvent *) xev;
+        XklFilterEvents (xevent);
+        return GDK_FILTER_CONTINUE;
+}
+
 void
 gnome_settings_keyboard_xkb_init (GConfClient * client)
 {
+#ifdef GSDKX
+	XklSetDebugLevel (200);
+	logfile = fopen ("/tmp/gsdkx.log", "a");
+        XklSetLogAppender (gnome_settings_keyboard_log_appender);
+#endif
+
 	if (!XklInit (GDK_DISPLAY ())) {
 		initedOk = TRUE;
 		XklBackupNamesProp ();
 		gnome_settings_keyboard_xkb_analyze_sysconfig ();
 		gnome_settings_keyboard_xkb_chk_lcl_xmm ();
+
 		gnome_settings_daemon_register_callback
-		    ("/desktop/gnome/peripherals/keyboard/xkb",
+		    (GSWITCHIT_CONFIG_DIR,
 		     (KeyCallbackFunc) apply_settings);
+
+		gnome_settings_daemon_register_callback
+		    (GSWITCHIT_KBD_CONFIG_DIR,
+		     (KeyCallbackFunc) apply_xkb_settings);
+
+		gdk_window_add_filter (NULL,
+				       (GdkFilterFunc) gnome_settings_keyboard_xkb_evt_filter,
+				       NULL);
+		gdk_window_add_filter (gdk_get_default_root_window(),
+				       (GdkFilterFunc) gnome_settings_keyboard_xkb_evt_filter,
+				       NULL);
+		XklStartListen (XKLL_MANAGE_LAYOUTS | XKLL_MANAGE_WINDOW_STATES);
 	}
 }
 
 void
 gnome_settings_keyboard_xkb_load (GConfClient * client)
 {
+	GConfClient *confClient;
+	confClient = gconf_client_get_default ();
+
+	GSwitchItConfigInit (&currentConfig, confClient);
 	apply_settings ();
+
+	GSwitchItKbdConfigInit (&currentKbdConfig, confClient);
+	apply_xkb_settings ();
+
+	g_object_unref (confClient);
 }
