@@ -99,12 +99,24 @@ static void
 execute (char *cmd, gboolean sync)
 {
 	gboolean retval;
+	gchar **argv;
+	gint argc;
+	
+	retval = FALSE;
 
-	if (sync != FALSE)
-		retval = g_spawn_command_line_sync
-			(cmd, NULL, NULL, NULL, NULL);
-	else
-		retval = g_spawn_command_line_async (cmd, NULL);
+	if (g_shell_parse_argv (cmd, &argc, &argv, NULL)) {
+		if (sync != FALSE) {
+			retval = g_spawn_sync (g_get_home_dir (), 
+			                       argv, NULL, G_SPAWN_SEARCH_PATH,
+			                       NULL, NULL, NULL, NULL, NULL, NULL);
+		}
+		else {
+			retval = g_spawn_async (g_get_home_dir (), 
+			                        argv, NULL, G_SPAWN_SEARCH_PATH,
+			                        NULL, NULL, NULL, NULL);
+		}
+		g_strfreev (argv);
+	}
 
 	if (retval == FALSE)
 	{
@@ -416,7 +428,7 @@ init_kbd (Acme *acme)
 
 	for (i = 0; i < PLAY_KEY; i++)
 	{
-		const char *tmp;
+		char *tmp;
 		Key *key;
 
 		gconf_client_notify_add (acme->conf_client,
@@ -425,15 +437,19 @@ init_kbd (Acme *acme)
 
 		tmp = gconf_client_get_string (acme->conf_client,
 				keys[i].gconf_key, NULL);
-		if (is_valid_shortcut (tmp) == FALSE)
+		if (!is_valid_shortcut (tmp)) {
+		        g_free (tmp);
 			continue;
+		}
 
 		key = g_new0 (Key, 1);
-		if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE)
+		if (!egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state))
 		{
+		        g_free (tmp);
 			g_free (key);
 			continue;
 		}
+		g_free (tmp);
 
 		keys[i].key = key;
 
@@ -442,7 +458,7 @@ init_kbd (Acme *acme)
 
 	for (i = PLAY_KEY; i < HANDLED_KEYS; i++)
 	{
-		const char *tmp;
+		char *tmp;
 		Key *key;
 
 		gconf_client_notify_add (acme->conf_client,
@@ -451,15 +467,19 @@ init_kbd (Acme *acme)
 
 		tmp = gconf_client_get_string (acme->conf_client,
 				keys[i].gconf_key, NULL);
-		if (is_valid_shortcut (tmp) == FALSE)
+		if (!is_valid_shortcut (tmp)) {
+		        g_free (tmp);
 			continue;
+		}
 
 		key = g_new0 (Key, 1);
 		if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE)
 		{
+		        g_free (tmp);
 			g_free (key);
 			continue;
 		}
+		g_free (tmp);
 
 		keys[i].key = key;
 
@@ -630,7 +650,7 @@ do_www_action (Acme *acme, const char *url)
 	}
 
 	if (url == NULL)
-		command = g_strdup_printf (string, "");
+		command = g_strdup_printf (string, "about:blank");
 	else
 		command = g_strdup_printf (string, url);
 
